(in-package :my-package)
(use-package :oli)

; Command to create a workplane referencing the axis of a cylinder or cone
(sd-defdialog 'ae_coaxial
  :dialog-title "AE auf Achse"
  :toolbox-button nil
  :variables
    '((C_FACE :selection (*sd-cylinder-seltype* *sd-cone-seltype*)
              :title "Curved Face"
              :prompt-text "Select cylindrical or conical face"
              :after-input (progn
               (if (sd-cylinder-p (sd-inq-geo-props c_face :dest-space :global))
                      (sd-set-variable-status 'W_DIR_PT :enable t :optional t)
                      (if W_AXIS
                        (sd-set-variable-status 'W_DIR_PT :enable nil :optional t)
                        (sd-set-variable-status 'W_DIR_PT :enable t :optional t)))
               (unless (getf (sd-inq-face-pnt c_face :u 0 :v 0 :dest-space :global) :coordinates)
                 (sd-set-variable-status 'W_DIR_PT :enable t :optional nil)
               )
               )
      )
      (U_AXIS :value-type :grouped-boolean
              :initial-value t
              :title "U Axis"
              :after-input
               (if (sd-cylinder-p (sd-inq-geo-props c_face :dest-space :global))
                      (sd-set-variable-status 'W_DIR_PT :enable t :optional t)
                      (sd-set-variable-status 'W_DIR_PT :enable t :optional t)))
      (V_AXIS :value-type :grouped-boolean
              :initial-value nil
              :title "V Axis"
              :after-input
               (if (sd-cylinder-p (sd-inq-geo-props c_face :dest-space :global))
                      (sd-set-variable-status 'W_DIR_PT :enable t :optional t)
                      (sd-set-variable-status 'W_DIR_PT :enable t :optional t)))
      (W_AXIS :value-type :grouped-boolean
              :title "W Axis"
              :initial-value nil
              :after-input
               (sd-set-variable-status 'W_DIR_PT :enable nil :optional t))
      (W_DIR_PT  :value-type :point-3d
                 :title "Pt on WP"
                 :prompt-text "Specify point on workplane"
                 :initial-value nil
                 :initial-optional t
                 :initial-enable t)
     )
  :mutual-exclusion '(U_AXIS V_AXIS W_AXIS)
  :start-variable 'C_FACE
  :ok-action
   '(let (face_axis secondpt startpt origin normaldir face-struct)
        (setq face-struct (sd-inq-geo-props c_face :dest-space :global))
        (cond ((sd-cylinder-p face-struct)
                (progn
                  (setq face_axis (sd-cylinder-axis-dir face-struct))
                  (setq normaldir (sd-cylinder-start-dir face-struct))
                  (setq startpt (getf (sd-inq-face-pnt c_face :u 0 :v 0 :dest-space :global) :coordinates))
                  (setq origin (sd-cylinder-center face-struct))))
              ((sd-cone-p face-struct)
                (progn
                  (setq face_axis (sd-cone-axis-dir face-struct))
                  (setq startpt (getf (sd-inq-face-pnt c_face :u 0 :v 0 :dest-space :global) :coordinates))
                  (setq origin (sd-cone-apex face-struct)))))
        (setq secondpt (sd-vec-add origin face_axis))
        (cond (U_AXIS
                (progn
                  (if W_DIR_PT
                    (sd-call-cmds (create_workplane :new :pt_pt_pt origin secondpt W_DIR_PT))
                    (sd-call-cmds (create_workplane :new :pt_pt_pt origin secondpt startpt)))
                  (sd-call-cmds (c_line_inf :horizontal 0))
                  (when (sd-cone-p face-struct)
                      (progn
                         (sd-call-cmds (c_line_inf :angle 0,0 (sd-cone-angle face-struct)))
                         (sd-call-cmds (c_line_inf :angle 0,0 (* -1 (sd-cone-angle face-struct))))))
                  (when (sd-cylinder-p face-struct)
                      (progn
                         (sd-call-cmds (c_line_inf :horizontal (sd-cylinder-radius face-struct)))
                         (sd-call-cmds (c_line_inf :horizontal (* -1 (sd-cylinder-radius face-struct))))))
                )
              )
              (V_AXIS
                (progn
                  (if W_DIR_PT
                    (sd-call-cmds (create_workplane :new :pt_pt_pt origin secondpt W_DIR_PT))
                    (sd-call-cmds (create_workplane :new :pt_pt_pt origin secondpt startpt)))
                  (sd-call-cmds (position_wp :current :rotate :axis :w :rotation_angle (* -1 (/ pi 2))))
                  (sd-call-cmds (c_line_inf :vertical 0))
                  (when (sd-cone-p face-struct)
                      (progn
                         (sd-call-cmds (c_line_inf :angle 0,0 (sd-cone-angle face-struct)))
                         (sd-call-cmds (c_line_inf :angle 0,0 (* -1 (sd-cone-angle face-struct))))))
                  (when (sd-cylinder-p face-struct)
                      (progn
                         (sd-call-cmds (c_line_inf :vertical (sd-cylinder-radius face-struct)))
                         (sd-call-cmds (c_line_inf :vertical (* -1 (sd-cylinder-radius face-struct)))))))
              )
              (W_AXIS
                (progn
                  (sd-call-cmds (create_workplane :new :pt_dir :origin origin :normal face_axis))
                   (sd-call-cmds (c_line_inf :horizontal 0))
                   (sd-call-cmds (c_line_inf :vertical 0))))
         )
    )
)

